# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import pytest

from aiopki.ext import keychain


@pytest.mark.asyncio
async def test_sign_using_default(keychain: keychain.Keychain):
    key = keychain.default()
    sig = await key.sign(b'Hello world!')
    assert await keychain.verify(sig, b'Hello world!', using=key.thumbprint)