# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import pydantic

from aiopki.types import ICryptoKey
from .cryptokeyuri import CryptoKeyURI


class CryptoKeySpecification(pydantic.BaseModel):
    __module__: str = 'aiopki'
    default: bool = False
    name: str
    uri: CryptoKeyURI
    using: str | None = None
    _key: ICryptoKey | None = None

    async def discover(self) -> ICryptoKey:
        if self._key is None:
            self._key = await self.uri
        return self._key

    def __await__(self):
        return self.discover().__await__()