# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import pytest_asyncio

from aiopki.ext.keychain import Keychain


@pytest_asyncio.fixture(scope='session') # type: ignore
async def keychain() -> Keychain:
    return await Keychain.model_validate({
        'keys': [
            {
                'name': 'ed448',
                'uri': 'file:///pki/sig.key?kty=OKP&crv=Ed448'
            }
        ]
    })

