# Copyright (C) 2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from canonical import HTTPResourceLocator

import aiopki
from aiopki.tests.signing import *


@pytest.fixture(scope='module')
def key_uri() -> HTTPResourceLocator:
    return HTTPResourceLocator.fromstring('https://python-oauthx.vault.azure.net/keys/ES256')


@pytest.fixture(scope='module')
def signing_algorithm() -> IAlgorithm:
    return aiopki.algorithms.get('ES256')